"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AccessoryFactory = void 0;
class AccessoryFactory {
    constructor(logger, homebridge, alexa, devicePredicates, serviceInitializers, characteristicInitializers) {
        this.logger = logger;
        this.homebridge = homebridge;
        this.alexa = alexa;
        this.devicePredicates = devicePredicates;
        this.serviceInitializers = serviceInitializers;
        this.characteristicInitializers = characteristicInitializers;
    }
    createAccessories() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield Promise.all((yield this.getDevices())
                .filter(device => this.devicePredicates.every(predicate => predicate.test(device)))
                .map(device => this.createAccessory(device)));
        });
    }
    createAccessory(device) {
        return __awaiter(this, void 0, void 0, function* () {
            const media = yield this.getMedia(device);
            const accessory = this.newAccessory(device);
            this.serviceInitializers.forEach(serviceInitializer => {
                const serviceType = serviceInitializer.getServiceType(device);
                if (serviceType) {
                    const service = this.getService(accessory, serviceType);
                    const characteristics = new Set(serviceInitializer.getCharacteristics(device).map(char => char.UUID));
                    this.characteristicInitializers
                        .filter(characteristicInitializer => characteristics.has(characteristicInitializer.getCharacteristic().UUID))
                        .forEach(characteristicInitializer => characteristicInitializer.initialize(service, device, media));
                }
            });
            return accessory;
        });
    }
    getDevices() {
        return new Promise((resolve, reject) => {
            this.alexa.getDevices((error, data) => {
                if (error) {
                    return reject(error);
                }
                resolve(data.devices);
            });
        });
    }
    getMedia(device) {
        return new Promise((resolve, reject) => {
            this.alexa.getMedia(device.serialNumber, (error, data) => {
                if (error) {
                    return reject(error);
                }
                resolve(data);
            });
        });
    }
    newAccessory(device) {
        const name = device.accountName;
        const serialNumber = device.serialNumber;
        const uuid = this.homebridge.hap.uuid.generate(serialNumber);
        const category = AccessoryFactory.TV_DEVICE_FAMILES.has(device.deviceFamily)
            ? 31 /* TELEVISION */
            : 26 /* SPEAKER */;
        this.logger.debug(`Creating accessory for device: ${JSON.stringify(device)}`);
        return new this.homebridge.platformAccessory(name, uuid, category);
    }
    getService(accessory, serviceType) {
        let service = accessory.getService(serviceType);
        if (!service) {
            service = accessory.addService(serviceType);
        }
        return service;
    }
}
exports.AccessoryFactory = AccessoryFactory;
AccessoryFactory.TV_DEVICE_FAMILES = new Set(['KNIGHT']);
//# sourceMappingURL=accessoryFactory.js.map