"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AlexaBridge = void 0;
class AlexaBridge {
    constructor(logger, hap, alexaRemote) {
        this.logger = logger;
        this.hap = hap;
        this.alexaRemote = alexaRemote;
        this.listeners = [];
    }
    onCharacteristicSet(device, service, characteristic, getCommand, getCommandValue) {
        service
            .getCharacteristic(characteristic)
            .on("set" /* SET */, (value, callback) => {
            try {
                const command = 'function' === typeof getCommand ? getCommand(value) : getCommand;
                const commandValue = 'function' === typeof getCommandValue ? getCommandValue(value) : getCommandValue;
                this.alexaRemote.sendCommand(device.serialNumber, command, commandValue, e => {
                    if (e) {
                        const message = `Failed to send ${command} to set ${characteristic.name} to ${JSON.stringify(commandValue)}.`;
                        this.logger.error(message, e);
                        return callback(new Error(message));
                    }
                    return callback(undefined);
                });
            }
            catch (e) {
                this.logger.error(`Uncaught error on setting ${characteristic.name}: ${JSON.stringify(e)}`);
                callback(e);
            }
        });
    }
    onDeviceCommand(device, listener) {
        if (0 === this.listeners.length) {
            this.alexaRemote.on('command', command => this.onCommand(command));
        }
        this.listeners.push({
            device,
            callback: listener,
        });
    }
    onCommand(command) {
        if (this.isDopplerCommand(command)) {
            this.listeners
                .filter(listener => listener.device.serialNumber === command.payload.dopplerId.deviceSerialNumber)
                .forEach(listener => listener.callback(command));
        }
    }
    isDopplerCommand(command) {
        return 'dopplerId' in command.payload;
    }
}
exports.AlexaBridge = AlexaBridge;
//# sourceMappingURL=alexaBridge.js.map