"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AlexaPlatformPlugin = void 0;
const fs = __importStar(require("fs"));
const process_1 = require("process");
const ajv_1 = __importDefault(require("ajv"));
const alexa_remote2_1 = __importDefault(require("alexa-remote2"));
const accessoryFactory_1 = require("./accessoryFactory");
const alexaBridge_1 = require("./alexaBridge");
const characteristicInitializer_1 = require("./characteristicInitializer");
const devicePredicate_1 = require("./devicePredicate");
const serviceInitializer_1 = require("./serviceInitializer");
class AlexaPlatformPlugin {
    constructor(logger, config, api) {
        var _a;
        const npmPackage = JSON.parse(fs.readFileSync(require.resolve('../package.json'), 'utf8'));
        logger.info(`Running ${npmPackage.name}-v${npmPackage.version} with homebridge-v${api.serverVersion} on node-${process_1.version}.`);
        try {
            if (this.validateConfig(this.getSchema(), config)) {
                const alexaRemote = new alexa_remote2_1.default();
                const alexaBridge = new alexaBridge_1.AlexaBridge(logger, api.hap, alexaRemote);
                const deviceFilters = [
                    new devicePredicate_1.DeviceFamilyPredicate(logger, ...AlexaPlatformPlugin.ALLOWED_DEVICE_FAMILIES),
                    new devicePredicate_1.DeviceCapabilitiesPredicate(logger, ...AlexaPlatformPlugin.REQUIRED_DEVICE_CAPABILTIES),
                ];
                const serviceInitializers = [
                    new serviceInitializer_1.AccessoryInfoServiceInitializer(api.hap, config),
                    new serviceInitializer_1.SmartSpeakerServiceInitializer(api.hap, config),
                    new serviceInitializer_1.TelevisionServiceInitializer(api.hap, config),
                    new serviceInitializer_1.TelevisionSpeakerServiceInitializer(api.hap, config),
                ];
                const characteristicInitializers = [
                    new characteristicInitializer_1.CurrentMediaStateInitializer(logger, api.hap, alexaBridge),
                    new characteristicInitializer_1.TargetMediaStateInitializer(logger, api.hap, alexaBridge),
                    new characteristicInitializer_1.MuteInitializer(logger, api.hap, alexaBridge),
                    new characteristicInitializer_1.VolumeInitializer(logger, api.hap, alexaBridge),
                    new characteristicInitializer_1.ManufacturerInitializer(logger, api.hap, alexaBridge),
                    new characteristicInitializer_1.ModelInitializer(logger, api.hap, alexaBridge),
                    new characteristicInitializer_1.FirmwareRevisionInitializer(logger, api.hap, alexaBridge),
                    new characteristicInitializer_1.NameInitializer(logger, api.hap, alexaBridge),
                    new characteristicInitializer_1.SerialNumberInitializer(logger, api.hap, alexaBridge),
                ];
                const accessoryFactory = new accessoryFactory_1.AccessoryFactory(logger, api, alexaRemote, deviceFilters, serviceInitializers, characteristicInitializers);
                alexaRemote.init({
                    useWsMqtt: true,
                    cookie: (_a = config.auth) === null || _a === void 0 ? void 0 : _a.cookie,
                    proxyOwnIp: config.auth.proxy.clientHost,
                    proxyPort: config.auth.proxy.port,
                    // TODO: Move the remaining entries to config…
                    alexaServiceHost: AlexaPlatformPlugin.SERVICE_HOST_DEFAULT,
                    amazonPage: AlexaPlatformPlugin.PROXY_SERVICE_HOST_DEFAULT,
                    amazonPageProxyLanguage: AlexaPlatformPlugin.PROXY_LANGUAGE,
                    acceptLanguage: AlexaPlatformPlugin.SERVICE_LANGUAGE,
                }, error => {
                    if (error) {
                        logger.error('Failed to initialize.', error);
                        return;
                    }
                    if (!config.auth.cookie && alexaRemote.cookie) {
                        logger.warn(`Alexa cookie retrieved successfully. Save this value in the Homebridge AlexaPlayer configuration as auth.cookie, but never share it with anyone: ${alexaRemote.cookie}`);
                    }
                    accessoryFactory
                        .createAccessories()
                        .then(accessories => {
                        api.publishExternalAccessories('homebridge-alexa-player', accessories);
                    })
                        .catch(err => {
                        logger.error(err);
                    });
                });
            }
        }
        catch (e) {
            logger.error(e);
        }
    }
    validateConfig(schema, config) {
        const ajv = new ajv_1.default();
        const isValid = ajv.validate(schema, config);
        if (!isValid && ajv.errors && 0 < ajv.errors.length) {
            const error = ajv.errors[0];
            const message = `Configuration error: config${error.dataPath} ${error.message || ''}`;
            throw new Error(message);
        }
        return true;
    }
    getSchema() {
        try {
            const schemaPath = require.resolve('../config.schema.json');
            const file = fs.readFileSync(schemaPath, 'utf8');
            const schema = JSON.parse(file);
            return schema.schema;
        }
        catch (e) {
            let message = 'Unable to read/parse configuration schema';
            if (this.hasMessage(e)) {
                message = `${message}: ${e.message}`;
            }
            throw new Error(message);
        }
    }
    hasMessage(error) {
        return 'object' === typeof error && null !== error && 'message' in error;
    }
}
exports.AlexaPlatformPlugin = AlexaPlatformPlugin;
// Example device families:
// - ECHO (echo)
// - TABLET (fire tablet)
// - MSHOP (mobile shopping app)
// - THIRD_PARTY_AVS_MEDIA_DISPLAY (Sonos)
// - THIRD_PARTY_AVS_SONOS_BOOTLEG (Sonos)
AlexaPlatformPlugin.ALLOWED_DEVICE_FAMILIES = ['ECHO', 'KNIGHT'];
// Speaker-relevant device capabilities:
// - VOLUME_SETTING (mShop, fire tablet, echo, Sonos)
// - SOUND_SETTINGS (fire tablet, echo)
// - AUDIO_CONTROLS (echo, Sonos Beam)
// - AUDIO_PLAYER (echo, Sonos)
// - DS_VOLUME_SETTING (echo)
AlexaPlatformPlugin.REQUIRED_DEVICE_CAPABILTIES = ['DS_VOLUME_SETTING', 'AUDIO_PLAYER'];
AlexaPlatformPlugin.PROXY_SERVICE_HOST_DEFAULT = 'amazon.com';
AlexaPlatformPlugin.PROXY_LANGUAGE = 'en_US';
AlexaPlatformPlugin.SERVICE_HOST_DEFAULT = 'pitangui.amazon.com';
AlexaPlatformPlugin.SERVICE_LANGUAGE = 'en-US';
//# sourceMappingURL=alexaPlatformPlugin.js.map