"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AccessoryInfoServiceInitializer = exports.TelevisionSpeakerServiceInitializer = exports.TelevisionServiceInitializer = exports.SmartSpeakerServiceInitializer = exports.BaseServiceInitializer = void 0;
class BaseServiceInitializer {
    constructor(hap, config) {
        this.hap = hap;
        this.config = config;
    }
    isTelevision(device) {
        return this.config.screensAsTelevisions === true && 'KNIGHT' === device.deviceFamily;
    }
}
exports.BaseServiceInitializer = BaseServiceInitializer;
class SmartSpeakerServiceInitializer extends BaseServiceInitializer {
    getServiceType(device) {
        return this.isTelevision(device) ? undefined : this.hap.Service.SmartSpeaker;
    }
    getCharacteristics() {
        return [
            this.hap.Characteristic.CurrentMediaState,
            this.hap.Characteristic.TargetMediaState,
            this.hap.Characteristic.Volume,
            this.hap.Characteristic.Mute,
        ];
    }
}
exports.SmartSpeakerServiceInitializer = SmartSpeakerServiceInitializer;
class TelevisionServiceInitializer extends BaseServiceInitializer {
    getServiceType(device) {
        return this.isTelevision(device) ? this.hap.Service.Television : undefined;
    }
    getCharacteristics() {
        return [this.hap.Characteristic.CurrentMediaState, this.hap.Characteristic.TargetMediaState];
    }
}
exports.TelevisionServiceInitializer = TelevisionServiceInitializer;
class TelevisionSpeakerServiceInitializer extends BaseServiceInitializer {
    getServiceType(device) {
        return this.isTelevision(device) ? this.hap.Service.TelevisionSpeaker : undefined;
    }
    getCharacteristics() {
        return [this.hap.Characteristic.Mute, this.hap.Characteristic.Volume];
    }
}
exports.TelevisionSpeakerServiceInitializer = TelevisionSpeakerServiceInitializer;
class AccessoryInfoServiceInitializer extends BaseServiceInitializer {
    getServiceType() {
        return this.hap.Service.AccessoryInformation;
    }
    getCharacteristics() {
        return [
            // TODO: hap.Characteristic.Identify
            this.hap.Characteristic.Manufacturer,
            this.hap.Characteristic.Model,
            this.hap.Characteristic.Name,
            this.hap.Characteristic.Model,
            this.hap.Characteristic.SerialNumber,
            this.hap.Characteristic.FirmwareRevision,
        ];
    }
}
exports.AccessoryInfoServiceInitializer = AccessoryInfoServiceInitializer;
//# sourceMappingURL=serviceInitializer.js.map